#!/usr/bin/python

import os
import sys
import time
import select
import serial

md5sumcmd="md5sum -c "+sys.argv[1]+".md5sum"
os.system(md5sumcmd)

MAX_CONNECT_COUNTER = 9
REBOOT_CMD = 'UPS-REBOOT\r'
ENTER_PWD = '!prgdata!'
TTY_DEVICE = '/dev/ttyS0'

PAD = chr(0xff) + chr(0xcf)

def readbin():
    try:
        bin = open(sys.argv[1]).read()
        print 'Opening firmware file.. ',
    except Exception, ex:
        print 'ERROR: CANNOT OPEN FIRMWARE FILE', ex
    
    try:
        main_page_size = len(bin) / 128
        rest_page_size = len(bin) - main_page_size * 128

        padding = ''
        if rest_page_size != 0:
            padding = PAD * ((128 - rest_page_size) / 2)
        
        if ((main_page_size + rest_page_size) % 256) != 0:
            padding += 64 * PAD
        
        print "Main/Rest/Pad Size: %d/%d/%d" % (main_page_size, rest_page_size, len(padding))
        
        nfile = sys.argv[1]+'_pad.bin'
        data = bin + padding
        f = open(nfile, 'w')
        f.write(data)
        return nfile
    except Exception, ex:
        print 'PADDING FILE ERROR', ex
        return False

def conn():
    global ser
    try:
        ser = serial.Serial(TTY_DEVICE, 19200, timeout=1)
    except Exception, ex:
        print 'CANNOT OPEN SERIAL PORT'
        return False
    ser.flushInput()
    ser.flushOutput()
    
    counter = 0
    while True:
        
        ack = ser.readline()
        #print repr(ack)
        
        if (ack.find('Mopower UPS Bootloader') == -1) and (ack.find('CCC') == -1):
            counter += 1
            if counter >= MAX_CONNECT_COUNTER:
                print 'device no respond'
                return False
            else:
                print '. \b',
                sys.stdout.flush()
        else:
            print 'CONNECTED, sending password..'
            ser.write(ENTER_PWD)
            ack = ser.readline()
            print ack
            return True
        
        ser.write(REBOOT_CMD)


def flash(bin):
    global ser
    
    ser.close()

    cmd = 'sz --xmodem -C3 -v "%s" < %s > %s' %(bin, TTY_DEVICE, TTY_DEVICE)
    child_stdin, child_stdout, child_stderr = os.popen3(cmd)
    child_stdin.close()

    star = ('|', '/', '-', '\\')
    idx = 0
    while (True):
        try:
            stdin, stdout, stderr = select.select((child_stdout, child_stderr), (), (), 0.5)
        except Exception, ex:
            print 'select exception: %s' %(ex)
            return False

        #val_out = ''
        #val_err = ''
        for fd in stdin:
            if fd == child_stdout:
                val_out = child_stdout.readline()
                print '[DEBUG] ->', repr(val_out)
            elif fd == child_stderr:
                val_err = child_stderr.readline()
                if val_err == '':
                    return False
                else:
                    print '[DEBUG] ->', repr(val_err)
        
        if val_err.find('Transfer complete') != -1:
            print 'DONE'
            return True
        else:
            print '%c\b\b'%(star[idx]),
            idx += 1
            idx %= len(star)
            sys.stdout.flush()
        
        time.sleep(0.1)


##########################
if __name__ == '__main__':
    bin = readbin()
    if (bin == False):
        print 'CANNOT ACCESS FILE'
        sys.exit(-1)
    
    if not conn():
        print 'CANNOT CONNECTED TO DEVICE'
        sys.exit(-1)
    
    if not flash(bin):
        print 'FLASHING ERROR'
        sys.exit(-1)
    else:
        print 'FLASHING DONE'
        sys.exit(0)

